#include "FxCyanF.hpp"

// C++ system level
#include <cmath>
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>
#include "SiliconTorch/FxCyanF.hpp"
#include "SiliconTorch/NVSExplorer.hpp"

// qthing stuff
// #include <qthing>



namespace SiliconTorch {

  namespace Service {

    void FxCyanF::init() {
      setName("FxCyanF");
      setNameSpace("fxCyan");  // TODO: "FxCyanF"
    }


    void FxCyanF::start() {

      const char* TAG = getName().c_str();

      if (fxCyan != NULL) {
        ESP_LOGW(TAG, "Service already running!");
        return;
      }


      fxCyan = new SiliconTorch::FxCyanF::FxCyanF();

      u32 frq = SiliconTorch::FxCyanF::DefaultFrequency;
      u8  res = SiliconTorch::FxCyanF::DefaultResolution;

      frq = NVSExplorer::NVSExplorer::instance().getUnsignedInt(getNameSpace(), "frequency", frq);
      res = NVSExplorer::NVSExplorer::instance().getUnsignedInt(getNameSpace(), "resolution", res);


      ESP_LOGI(TAG, "Configuring: FxCyanF{ frequency[ %d Hz ]  resolution[ %d Bit ] }", frq, res);
      fxCyan->setFrqRes(frq, res);


      for (u8 ch = 0; ch < SiliconTorch::FxCyanF::MAX_CHANNELS; ch++) {

        char buffer[32];
        snprintf(buffer, 32, "ch%d_gpio", ch);

        u8 gpio = NVSExplorer::NVSExplorer::instance().getUnsignedInt(getNameSpace(), buffer, 0xFF) & 0xFF;

        snprintf(buffer, 32, "ch%d_pwm", ch);
        f32 pwm = NVSExplorer::NVSExplorer::instance().getFloat(getNameSpace(), buffer);

        // NaN == "no value stored" or any other reading error
        if (std::isnan(pwm)) pwm = 0.0f;

        if (gpio != 0xFF) {  // key exists

          if ( fxCyan->addChannel(gpio, pwm) ) {  // channel creation successful
  
            ESP_LOGI(TAG, "Configuring: FxCyanF{ channel[ %d ]  gpio[ %d ]  pwm[ %f ] }", ch, gpio, pwm);

            fxCyan->setPWM(ch, pwm);

          } else {
            ESP_LOGW(TAG, "Configuring: FxCyanF{ channel[ %d ]  gpio[ %d ] }  FAILED[❌]", ch, gpio);
            break;
          }

        } else {
          break;
        }
      }


    }


    SiliconTorch::FxCyanF::FxCyanF* FxCyanF::getFxCyanF() const {
      return fxCyan;
    }



  }
}
